@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2008 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: tm2unixtime.c,v 1.13.2.3.2.2 2007/01/01 09:35:59 sebastian Exp $ */
+/* $Id: tm2unixtime.c,v 1.13.2.3.2.6 2008/10/26 11:27:32 derick Exp $ */
 
 #include "timelib.h"
 
@@ -98,6 +98,7 @@ static void do_adjust_for_weekday(timelib_time* time)
 	} else {
 		time->d -= (7 - (abs(time->relative.weekday) - current_dow));
 	}
+	time->have_weekday_relative = 0;
 }
 
 static void do_normalize(timelib_time* time)
@@ -130,39 +131,60 @@ static void do_adjust_relative(timelib_time* time)
 	do_normalize(time);
 
 	memset(&(time->relative), 0, sizeof(time->relative));
+	time->have_relative = 0;
 }
 
 static void do_adjust_special_weekday(timelib_time* time)
 {
-	timelib_sll current_dow, this_weekday = 0, count;
+	timelib_sll current_dow, count;
+
+	count = time->special.amount;
 
 	current_dow = timelib_day_of_week(time->y, time->m, time->d);
-	count = time->special.amount;
 	if (count == 0) {
+		/* skip over saturday and sunday */
 		if (current_dow == 6) {
-			this_weekday = 2;
+			time->d += 2;
 		}
+		/* skip over sunday */
 		if (current_dow == 0) {
-			this_weekday = 1;
+			time->d += 1;
 		}
-		time->d += this_weekday;
-		return;
 	} else if (count > 0) {
+		/* skip over saturday and sunday */
 		if (current_dow == 5) {
-			this_weekday = 2;
+			time->d += 2;
 		}
+		/* skip over sunday */
 		if (current_dow == 6) {
-			this_weekday = 1;
+			time->d += 1;
+		}
+		/* add increments of 5 weekdays as a week */
+		time->d += (count / 5) * 7;
+		/* if current DOW plus the remainder > 5, add two days */
+		current_dow = timelib_day_of_week(time->y, time->m, time->d);
+		time->d += (count % 5);
+		if ((count % 5) + current_dow > 5) {
+			time->d += 2;
 		}
 	} else if (count < 0) {
-		if (current_dow == 0) {
-			this_weekday = -1;
-		}
+		/* skip over sunday and saturday */
 		if (current_dow == 1) {
-			this_weekday = -2;
+			time->d -= 2;
+		}
+		/* skip over satruday */
+		if (current_dow == 0 ) {
+			time->d -= 1;
+		}
+		/* subtract increments of 5 weekdays as a week */
+		time->d += (count / 5) * 7;
+		/* if current DOW minus the remainder < 0, subtract two days */
+		current_dow = timelib_day_of_week(time->y, time->m, time->d);
+		time->d += (count % 5);
+		if ((count % 5) + current_dow < 1) {
+			time->d -= 2;
 		}
 	}
-	time->d += this_weekday + ((count / 5) * 7) + (count % 5);
 }
 
 static void do_adjust_special(timelib_time* time)
@@ -176,6 +198,7 @@ static void do_adjust_special(timelib_time* time)
 	}
 	do_normalize(time);
 	memset(&(time->special), 0, sizeof(time->special));
+	time->have_relative = 0;
 }
 
 static timelib_sll do_years(timelib_sll year)
@@ -184,10 +207,10 @@ static timelib_sll do_years(timelib_sll year)
 	timelib_sll res = 0;
 	timelib_sll eras;
 
-	eras = (year - 1970) / 400;
+	eras = (year - 1970) / 40000;
 	if (eras != 0) {
-		year = year - (eras * 400);
-		res += (SECS_PER_ERA * eras);
+		year = year - (eras * 40000);
+		res += (SECS_PER_ERA * eras * 100);
 	}
 
 	if (year >= 1970) {
