@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2005 The PHP Group                                |
+   | Copyright (c) 1997-2008 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: document.c,v 1.68.2.2 2005/09/08 10:39:30 rrichards Exp $ */
+/* $Id: document.c,v 1.68.2.3.2.9 2008/09/28 12:01:21 rrichards Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -41,6 +41,182 @@ struct _idsIterator {
 #define DOM_LOAD_STRING 0
 #define DOM_LOAD_FILE 1
 
+/* {{{ arginfo */
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_element, 0, 0, 1)
+	ZEND_ARG_INFO(0, tagName)
+	ZEND_ARG_INFO(0, value)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_document_fragment, 0, 0, 0)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_text_node, 0, 0, 1)
+	ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_comment, 0, 0, 1)
+	ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_cdatasection, 0, 0, 1)
+	ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_processing_instruction, 0, 0, 2)
+	ZEND_ARG_INFO(0, target)
+	ZEND_ARG_INFO(0, data)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_attribute, 0, 0, 1)
+	ZEND_ARG_INFO(0, name)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_entity_reference, 0, 0, 1)
+	ZEND_ARG_INFO(0, name)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_get_elements_by_tag_name, 0, 0, 1)
+	ZEND_ARG_INFO(0, tagName)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_import_node, 0, 0, 2)
+	ZEND_ARG_OBJ_INFO(0, importedNode, DOMNode, 0)
+	ZEND_ARG_INFO(0, deep)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_element_ns, 0, 0, 2)
+	ZEND_ARG_INFO(0, namespaceURI)
+	ZEND_ARG_INFO(0, qualifiedName)
+	ZEND_ARG_INFO(0, value)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_create_attribute_ns, 0, 0, 2)
+	ZEND_ARG_INFO(0, namespaceURI)
+	ZEND_ARG_INFO(0, qualifiedName)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_get_elements_by_tag_name_ns, 0, 0, 2)
+	ZEND_ARG_INFO(0, namespaceURI)
+	ZEND_ARG_INFO(0, localName)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_get_element_by_id, 0, 0, 1)
+	ZEND_ARG_INFO(0, elementId)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_adopt_node, 0, 0, 1)
+	ZEND_ARG_OBJ_INFO(0, source, DOMNode, 0)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_normalize_document, 0, 0, 0)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_rename_node, 0, 0, 3)
+	ZEND_ARG_OBJ_INFO(0, node, DOMNode, 0)
+	ZEND_ARG_INFO(0, namespaceURI)
+	ZEND_ARG_INFO(0, qualifiedName)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_load, 0, 0, 1)
+	ZEND_ARG_INFO(0, source)
+	ZEND_ARG_INFO(0, options)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_save, 0, 0, 1)
+	ZEND_ARG_INFO(0, file)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_loadxml, 0, 0, 1)
+	ZEND_ARG_INFO(0, source)
+	ZEND_ARG_INFO(0, options)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_savexml, 0, 0, 0)
+	ZEND_ARG_OBJ_INFO(0, node, DOMNode, 1)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_construct, 0, 0, 0)
+	ZEND_ARG_INFO(0, version)
+	ZEND_ARG_INFO(0, encoding)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_validate, 0, 0, 0)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_xinclude, 0, 0, 0)
+	ZEND_ARG_INFO(0, options)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_loadhtml, 0, 0, 1)
+	ZEND_ARG_INFO(0, source)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_loadhtmlfile, 0, 0, 1)
+	ZEND_ARG_INFO(0, source)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_savehtml, 0, 0, 0)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_savehtmlfile, 0, 0, 1)
+	ZEND_ARG_INFO(0, file)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_schema_validate_file, 0, 0, 1)
+	ZEND_ARG_INFO(0, filename)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_schema_validate_xml, 0, 0, 1)
+	ZEND_ARG_INFO(0, source)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_relaxNG_validate_file, 0, 0, 1)
+	ZEND_ARG_INFO(0, filename)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_relaxNG_validate_xml, 0, 0, 1)
+	ZEND_ARG_INFO(0, source)
+ZEND_END_ARG_INFO();
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_dom_document_registernodeclass, 0, 0, 2)
+	ZEND_ARG_INFO(0, baseClass)
+	ZEND_ARG_INFO(0, extendedClass)
+ZEND_END_ARG_INFO();
+/* }}} */
+
 /*
 * class DOMDocument extends DOMNode 
 *
@@ -49,42 +225,43 @@ struct _idsIterator {
 */
 
 zend_function_entry php_dom_document_class_functions[] = {
-	PHP_FALIAS(createElement, dom_document_create_element, NULL)
-	PHP_FALIAS(createDocumentFragment, dom_document_create_document_fragment, NULL)
-	PHP_FALIAS(createTextNode, dom_document_create_text_node, NULL)
-	PHP_FALIAS(createComment, dom_document_create_comment, NULL)
-	PHP_FALIAS(createCDATASection, dom_document_create_cdatasection, NULL)
-	PHP_FALIAS(createProcessingInstruction, dom_document_create_processing_instruction, NULL)
-	PHP_FALIAS(createAttribute, dom_document_create_attribute, NULL)
-	PHP_FALIAS(createEntityReference, dom_document_create_entity_reference, NULL)
-	PHP_FALIAS(getElementsByTagName, dom_document_get_elements_by_tag_name, NULL)
-	PHP_FALIAS(importNode, dom_document_import_node, NULL)
-	PHP_FALIAS(createElementNS, dom_document_create_element_ns, NULL)
-	PHP_FALIAS(createAttributeNS, dom_document_create_attribute_ns, NULL)
-	PHP_FALIAS(getElementsByTagNameNS, dom_document_get_elements_by_tag_name_ns, NULL)
-	PHP_FALIAS(getElementById, dom_document_get_element_by_id, NULL)
-	PHP_FALIAS(adoptNode, dom_document_adopt_node, NULL)
-	PHP_FALIAS(normalizeDocument, dom_document_normalize_document, NULL)
-	PHP_FALIAS(renameNode, dom_document_rename_node, NULL)
-	PHP_ME(domdocument, load, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_FALIAS(save, dom_document_save, NULL)
-	PHP_ME(domdocument, loadXML, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_FALIAS(saveXML, dom_document_savexml, NULL)
-	PHP_ME(domdocument, __construct, NULL, ZEND_ACC_PUBLIC)
-	PHP_FALIAS(validate, dom_document_validate, NULL)
-	PHP_FALIAS(xinclude, dom_document_xinclude, NULL)
+	PHP_FALIAS(createElement, dom_document_create_element, arginfo_dom_document_create_element)
+	PHP_FALIAS(createDocumentFragment, dom_document_create_document_fragment, arginfo_dom_document_create_document_fragment)
+	PHP_FALIAS(createTextNode, dom_document_create_text_node, arginfo_dom_document_create_text_node)
+	PHP_FALIAS(createComment, dom_document_create_comment, arginfo_dom_document_create_comment)
+	PHP_FALIAS(createCDATASection, dom_document_create_cdatasection, arginfo_dom_document_create_cdatasection)
+	PHP_FALIAS(createProcessingInstruction, dom_document_create_processing_instruction, arginfo_dom_document_create_processing_instruction)
+	PHP_FALIAS(createAttribute, dom_document_create_attribute, arginfo_dom_document_create_attribute)
+	PHP_FALIAS(createEntityReference, dom_document_create_entity_reference, arginfo_dom_document_create_entity_reference)
+	PHP_FALIAS(getElementsByTagName, dom_document_get_elements_by_tag_name, arginfo_dom_document_get_elements_by_tag_name)
+	PHP_FALIAS(importNode, dom_document_import_node, arginfo_dom_document_import_node)
+	PHP_FALIAS(createElementNS, dom_document_create_element_ns, arginfo_dom_document_create_element_ns)
+	PHP_FALIAS(createAttributeNS, dom_document_create_attribute_ns, arginfo_dom_document_create_attribute_ns)
+	PHP_FALIAS(getElementsByTagNameNS, dom_document_get_elements_by_tag_name_ns, arginfo_dom_document_get_elements_by_tag_name_ns)
+	PHP_FALIAS(getElementById, dom_document_get_element_by_id, arginfo_dom_document_get_element_by_id)
+	PHP_FALIAS(adoptNode, dom_document_adopt_node, arginfo_dom_document_adopt_node)
+	PHP_FALIAS(normalizeDocument, dom_document_normalize_document, arginfo_dom_document_normalize_document)
+	PHP_FALIAS(renameNode, dom_document_rename_node, arginfo_dom_document_rename_node)
+	PHP_ME(domdocument, load, arginfo_dom_document_load, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	PHP_FALIAS(save, dom_document_save, arginfo_dom_document_save)
+	PHP_ME(domdocument, loadXML, arginfo_dom_document_loadxml, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	PHP_FALIAS(saveXML, dom_document_savexml, arginfo_dom_document_savexml)
+	PHP_ME(domdocument, __construct, arginfo_dom_document_construct, ZEND_ACC_PUBLIC)
+	PHP_FALIAS(validate, dom_document_validate, arginfo_dom_document_validate)
+	PHP_FALIAS(xinclude, dom_document_xinclude, arginfo_dom_document_xinclude)
 #if defined(LIBXML_HTML_ENABLED)
-	PHP_ME(domdocument, loadHTML, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_ME(domdocument, loadHTMLFile, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
-	PHP_FALIAS(saveHTML, dom_document_save_html, NULL)
-	PHP_FALIAS(saveHTMLFile, dom_document_save_html_file, NULL)
+	PHP_ME(domdocument, loadHTML, arginfo_dom_document_loadhtml, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	PHP_ME(domdocument, loadHTMLFile, arginfo_dom_document_loadhtmlfile, ZEND_ACC_PUBLIC|ZEND_ACC_ALLOW_STATIC)
+	PHP_FALIAS(saveHTML, dom_document_save_html, arginfo_dom_document_savehtml)
+	PHP_FALIAS(saveHTMLFile, dom_document_save_html_file, arginfo_dom_document_savehtmlfile)
 #endif  /* defined(LIBXML_HTML_ENABLED) */
 #if defined(LIBXML_SCHEMAS_ENABLED)
-	PHP_FALIAS(schemaValidate, dom_document_schema_validate_file, NULL)
-	PHP_FALIAS(schemaValidateSource, dom_document_schema_validate_xml, NULL)
-	PHP_FALIAS(relaxNGValidate, dom_document_relaxNG_validate_file, NULL)
-	PHP_FALIAS(relaxNGValidateSource, dom_document_relaxNG_validate_xml, NULL)
+	PHP_FALIAS(schemaValidate, dom_document_schema_validate_file, arginfo_dom_document_schema_validate_file)
+	PHP_FALIAS(schemaValidateSource, dom_document_schema_validate_xml, arginfo_dom_document_schema_validate_xml)
+	PHP_FALIAS(relaxNGValidate, dom_document_relaxNG_validate_file, arginfo_dom_document_relaxNG_validate_file)
+	PHP_FALIAS(relaxNGValidateSource, dom_document_relaxNG_validate_xml, arginfo_dom_document_relaxNG_validate_xml)
 #endif
+	PHP_ME(domdocument, registerNodeClass, arginfo_dom_document_registernodeclass, ZEND_ACC_PUBLIC)
 	{NULL, NULL, NULL}
 };
 
@@ -384,7 +561,7 @@ Since: DOM Level 3
 */
 int dom_document_strict_error_checking_read(dom_object *obj, zval **retval TSRMLS_DC)
 {
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	ALLOC_ZVAL(*retval);
 	if (obj->document) {
@@ -399,7 +576,7 @@ int dom_document_strict_error_checking_read(dom_object *obj, zval **retval TSRML
 int dom_document_strict_error_checking_write(dom_object *obj, zval *newval TSRMLS_DC)
 {
 	zval value_copy;
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	if(newval->refcount > 1) {
 		value_copy = *newval;
@@ -427,7 +604,7 @@ readonly=no
 */
 int dom_document_format_output_read(dom_object *obj, zval **retval TSRMLS_DC)
 {
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	ALLOC_ZVAL(*retval);
 	if (obj->document) {
@@ -442,7 +619,7 @@ int dom_document_format_output_read(dom_object *obj, zval **retval TSRMLS_DC)
 int dom_document_format_output_write(dom_object *obj, zval *newval TSRMLS_DC)
 {
 	zval value_copy;
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	if(newval->refcount > 1) {
 		value_copy = *newval;
@@ -469,7 +646,7 @@ readonly=no
 */
 int	dom_document_validate_on_parse_read(dom_object *obj, zval **retval TSRMLS_DC)
 {
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	ALLOC_ZVAL(*retval);
 	if (obj->document) {
@@ -484,7 +661,7 @@ int	dom_document_validate_on_parse_read(dom_object *obj, zval **retval TSRMLS_DC
 int dom_document_validate_on_parse_write(dom_object *obj, zval *newval TSRMLS_DC)
 {
 	zval value_copy;
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	if(newval->refcount > 1) {
 		value_copy = *newval;
@@ -512,7 +689,7 @@ readonly=no
 */
 int dom_document_resolve_externals_read(dom_object *obj, zval **retval TSRMLS_DC)
 {
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	ALLOC_ZVAL(*retval);
 	if (obj->document) {
@@ -527,7 +704,7 @@ int dom_document_resolve_externals_read(dom_object *obj, zval **retval TSRMLS_DC
 int dom_document_resolve_externals_write(dom_object *obj, zval *newval TSRMLS_DC)
 {
 	zval value_copy;
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	if(newval->refcount > 1) {
 		value_copy = *newval;
@@ -555,7 +732,7 @@ readonly=no
 */
 int dom_document_preserve_whitespace_read(dom_object *obj, zval **retval TSRMLS_DC)
 {
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	ALLOC_ZVAL(*retval);
 	if (obj->document) {
@@ -570,7 +747,7 @@ int dom_document_preserve_whitespace_read(dom_object *obj, zval **retval TSRMLS_
 int dom_document_preserve_whitespace_write(dom_object *obj, zval *newval TSRMLS_DC)
 {
 	zval value_copy;
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	if(newval->refcount > 1) {
 		value_copy = *newval;
@@ -597,7 +774,7 @@ readonly=no
 */
 int dom_document_recover_read(dom_object *obj, zval **retval TSRMLS_DC)
 {
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	ALLOC_ZVAL(*retval);
 	if (obj->document) {
@@ -612,7 +789,7 @@ int dom_document_recover_read(dom_object *obj, zval **retval TSRMLS_DC)
 int dom_document_recover_write(dom_object *obj, zval *newval TSRMLS_DC)
 {
 	zval value_copy;
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	if(newval->refcount > 1) {
 		value_copy = *newval;
@@ -640,7 +817,7 @@ readonly=no
 */
 int dom_document_substitue_entities_read(dom_object *obj, zval **retval TSRMLS_DC)
 {
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	ALLOC_ZVAL(*retval);
 	if (obj->document) {
@@ -655,7 +832,7 @@ int dom_document_substitue_entities_read(dom_object *obj, zval **retval TSRMLS_D
 int dom_document_substitue_entities_write(dom_object *obj, zval *newval TSRMLS_DC)
 {
 	zval value_copy;
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 
 	if(newval->refcount > 1) {
 		value_copy = *newval;
@@ -1354,7 +1531,9 @@ PHP_METHOD(domdocument, __construct)
 			}
 		}
 		intern->document = NULL;
-		php_libxml_increment_doc_ref((php_libxml_node_object *)intern, docp TSRMLS_CC);
+		if (php_libxml_increment_doc_ref((php_libxml_node_object *)intern, docp TSRMLS_CC) == -1) {
+			RETURN_FALSE;
+		}
 		php_libxml_increment_node_ptr((php_libxml_node_object *)intern, (xmlNodePtr)docp, (void *)intern TSRMLS_CC);
 	}
 }
@@ -1394,8 +1573,9 @@ char *_dom_get_valid_file_path(char *source, char *resolved_path, int resolved_p
 
 	if ((uri->scheme == NULL || isFileUri)) {
 		/* XXX possible buffer overflow if VCWD_REALPATH does not know size of resolved_path */
-		if (! VCWD_REALPATH(source, resolved_path)) {
-			expand_filepath(source, resolved_path TSRMLS_CC);
+		if (!VCWD_REALPATH(source, resolved_path) && !expand_filepath(source, resolved_path TSRMLS_CC)) {
+			xmlFreeURI(uri);
+			return NULL;
 		}
 		file_dest = resolved_path;
 	}
@@ -1410,12 +1590,12 @@ char *_dom_get_valid_file_path(char *source, char *resolved_path, int resolved_p
 static xmlDocPtr dom_document_parser(zval *id, int mode, char *source, int options TSRMLS_DC) {
     xmlDocPtr ret;
     xmlParserCtxtPtr ctxt = NULL;
-	dom_doc_props *doc_props;
+	dom_doc_propsptr doc_props;
 	dom_object *intern;
 	php_libxml_ref_obj *document = NULL;
 	int validate, recover, resolve_externals, keep_blanks, substitute_ent;
 	int resolved_path_len;
-	int old_error_reporting;
+	int old_error_reporting = 0;
 	char *directory=NULL, resolved_path[MAXPATHLEN];
 
 	if (id != NULL) {
@@ -1436,10 +1616,6 @@ static xmlDocPtr dom_document_parser(zval *id, int mode, char *source, int optio
 
 	xmlInitParser();
 
-#if LIBXML_VERSION < 20600
-	keep_blanks = xmlKeepBlanksDefault(keep_blanks);
-#endif
-
 	if (mode == DOM_LOAD_FILE) {
 		char *file_dest = _dom_get_valid_file_path(source, resolved_path, MAXPATHLEN  TSRMLS_CC);
 		if (file_dest) {
@@ -1450,14 +1626,6 @@ static xmlDocPtr dom_document_parser(zval *id, int mode, char *source, int optio
 		ctxt = xmlCreateDocParserCtxt(source);
 	}
 
-#if LIBXML_VERSION < 20600
-	xmlKeepBlanksDefault(keep_blanks);
-	/* xmlIndentTreeOutput default is changed in xmlKeepBlanksDefault
-	reset back to 1 which is default value */
-
-	xmlIndentTreeOutput = 1;
-#endif
-
 	if (ctxt == NULL) {
 		return(NULL);
 	}
@@ -1490,7 +1658,6 @@ static xmlDocPtr dom_document_parser(zval *id, int mode, char *source, int optio
 		ctxt->sax->warning = php_libxml_ctx_warning;
 	}
 
-#if LIBXML_VERSION >= 20600
 	if (validate && ! (options & XML_PARSE_DTDVALID)) {
 		options |= XML_PARSE_DTDVALID;
 	}
@@ -1505,11 +1672,6 @@ static xmlDocPtr dom_document_parser(zval *id, int mode, char *source, int optio
 	}
 
 	xmlCtxtUseOptions(ctxt, options);
-#else
-	ctxt->validate = validate;
-    ctxt->loadsubset = (resolve_externals * XML_COMPLETE_ATTRS);
-	ctxt->replaceEntities = substitute_ent;
-#endif
 
 	ctxt->recovery = recover;
 	if (recover) {
@@ -1544,7 +1706,7 @@ static xmlDocPtr dom_document_parser(zval *id, int mode, char *source, int optio
 static void dom_parse_document(INTERNAL_FUNCTION_PARAMETERS, int mode) {
 	zval *id, *rv = NULL;
 	xmlDoc *docp = NULL, *newdoc;
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 	dom_object *intern;
 	char *source;
 	int source_len, refcount, ret;
@@ -1584,7 +1746,9 @@ static void dom_parse_document(INTERNAL_FUNCTION_PARAMETERS, int mode) {
 				}
 			}
 			intern->document = NULL;
-			php_libxml_increment_doc_ref((php_libxml_node_object *)intern, newdoc TSRMLS_CC);
+			if (php_libxml_increment_doc_ref((php_libxml_node_object *)intern, newdoc TSRMLS_CC) == -1) {
+				RETURN_FALSE;
+			}
 			intern->document->doc_props = doc_prop;
 		}
 
@@ -1624,9 +1788,9 @@ PHP_FUNCTION(dom_document_save)
 {
 	zval *id;
 	xmlDoc *docp;
-	int file_len = 0, bytes, format, saveempty;
+	int file_len = 0, bytes, format, saveempty = 0;
 	dom_object *intern;
-	dom_doc_props *doc_props;
+	dom_doc_propsptr doc_props;
 	char *file;
 	long options = 0;
 
@@ -1672,8 +1836,8 @@ PHP_FUNCTION(dom_document_savexml)
 	xmlBufferPtr buf;
 	xmlChar *mem;
 	dom_object *intern, *nodeobj;
-	dom_doc_props *doc_props;
-	int size, format, saveempty;
+	dom_doc_propsptr doc_props;
+	int size, format, saveempty = 0;
 	long options = 0;
 
 	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "O|O!l", &id, dom_document_class_entry, &nodep, dom_node_class_entry, &options) == FAILURE) {
@@ -1749,6 +1913,10 @@ static void php_dom_remove_xinclude_nodes(xmlNodePtr cur TSRMLS_DC) {
 
 			/* XML_XINCLUDE_END node will be a sibling of XML_XINCLUDE_START */
 			while(cur && cur->type != XML_XINCLUDE_END) {
+				/* remove xinclude processing nodes from recursive xincludes */
+				if (cur->type == XML_ELEMENT_NODE) {
+					   php_dom_remove_xinclude_nodes(cur->children TSRMLS_CC);
+				}
 				cur = cur->next;
 			}
 
@@ -1781,11 +1949,7 @@ PHP_FUNCTION(dom_document_xinclude)
 
 	DOM_GET_OBJ(docp, id, xmlDocPtr, intern);
 
-#if LIBXML_VERSION >= 20607
 	err = xmlXIncludeProcessFlags(docp, flags);
-#else
-	err = xmlXIncludeProcess (docp);
-#endif
 
 	/* XML_XINCLUDE_START and XML_XINCLUDE_END nodes need to be removed as these
 	are added via xmlXIncludeProcess to mark beginning and ending of xincluded document 
@@ -2034,7 +2198,7 @@ static void dom_load_html(INTERNAL_FUNCTION_PARAMETERS, int mode)
 	zval *id, *rv = NULL;
 	xmlDoc *docp = NULL, *newdoc;
 	dom_object *intern;
-	dom_doc_props *doc_prop;
+	dom_doc_propsptr doc_prop;
 	char *source;
 	int source_len, refcount, ret;
 	htmlParserCtxtPtr ctxt;
@@ -2089,7 +2253,9 @@ static void dom_load_html(INTERNAL_FUNCTION_PARAMETERS, int mode)
 				}
 			}
 			intern->document = NULL;
-			php_libxml_increment_doc_ref((php_libxml_node_object *)intern, newdoc TSRMLS_CC);
+			if (php_libxml_increment_doc_ref((php_libxml_node_object *)intern, newdoc TSRMLS_CC) == -1) {
+				RETURN_FALSE;
+			}
 			intern->document->doc_props = doc_prop;
 		}
 
@@ -2128,7 +2294,7 @@ PHP_FUNCTION(dom_document_save_html_file)
 	xmlDoc *docp;
 	int file_len, bytes, format;
 	dom_object *intern;
-	dom_doc_props *doc_props;
+	dom_doc_propsptr doc_props;
 	char *file;
 
 	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Os", &id, dom_document_class_entry, &file, &file_len) == FAILURE) {
@@ -2185,4 +2351,57 @@ PHP_FUNCTION(dom_document_save_html)
 
 #endif  /* defined(LIBXML_HTML_ENABLED) */
 
+/* {{{ proto boolean DOMDocument::registerNodeClass(string baseclass, string extendedclass);
+   Register extended class used to create base node type */
+PHP_METHOD(domdocument, registerNodeClass)
+{
+	zval *id;
+	xmlDoc *docp;
+	char *baseclass = NULL, *extendedclass = NULL;
+	int baseclass_len = 0, extendedclass_len = 0;
+	zend_class_entry *basece = NULL, *ce = NULL;
+	dom_object *intern;
+
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Oss!", &id, dom_document_class_entry, &baseclass, &baseclass_len, &extendedclass, &extendedclass_len) == FAILURE) {
+		return;
+	}
+
+	if (baseclass_len) {
+		zend_class_entry **pce;
+		if (zend_lookup_class(baseclass, baseclass_len, &pce TSRMLS_CC) == FAILURE) {
+			php_error_docref(NULL TSRMLS_CC, E_ERROR, "Class %s does not exist", baseclass);
+			return;
+		}
+		basece = *pce;
+	}
+
+	if (basece == NULL || ! instanceof_function(basece, dom_node_class_entry TSRMLS_CC)) {
+		php_error_docref(NULL TSRMLS_CC, E_ERROR, "Class %s is not derived from DOMNode.", baseclass);
+		return;
+	}
+
+	if (extendedclass_len) {
+		zend_class_entry **pce;
+		if (zend_lookup_class(extendedclass, extendedclass_len, &pce TSRMLS_CC) == FAILURE) {
+			php_error_docref(NULL TSRMLS_CC, E_ERROR, "Class %s does not exist", extendedclass);
+		}
+		ce = *pce;
+	}
+
+	if (ce == NULL || instanceof_function(ce, basece TSRMLS_CC)) {
+
+		DOM_GET_OBJ(docp, id, xmlDocPtr, intern);
+
+		if (dom_set_doc_classmap(intern->document, basece, ce TSRMLS_CC) == FAILURE) {
+			php_error_docref(NULL TSRMLS_CC, E_ERROR, "Class %s could not be registered.", extendedclass);
+		}
+		RETURN_TRUE;
+	} else {
+		php_error_docref(NULL TSRMLS_CC, E_ERROR, "Class %s is not derived from %s.", extendedclass, baseclass);
+	}
+
+	RETURN_FALSE;
+}
+/* }}} */
+
 #endif  /* HAVE_LIBXML && HAVE_DOM */
