@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2005 The PHP Group                                |
+   | Copyright (c) 1997-2008 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: dom_iterators.c,v 1.9 2005/08/03 14:07:01 sniper Exp $ */
+/* $Id: dom_iterators.c,v 1.9.2.3.2.6 2007/12/31 07:20:05 sebastian Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -162,7 +162,7 @@ static int php_dom_iterator_current_key(zend_object_iterator *iter, char **str_k
 	object = (zval *)iterator->intern.data;
 
 	if (instanceof_function(Z_OBJCE_P(object), dom_nodelist_class_entry TSRMLS_CC)) {
-		*int_key = iter->index - 1;
+		*int_key = iter->index;
 		return HASH_KEY_IS_LONG;
 	} else {
 		curobj = iterator->curobj;
@@ -170,6 +170,8 @@ static int php_dom_iterator_current_key(zend_object_iterator *iter, char **str_k
 		intern = (dom_object *)zend_object_store_get_object(curobj TSRMLS_CC);
 		if (intern != NULL && intern->ptr != NULL) {
 			curnode = (xmlNodePtr)((php_libxml_node_ptr *)intern->ptr)->node;
+		} else {
+			return HASH_KEY_NON_EXISTANT;
 		}
 
 		namelen = xmlStrlen(curnode->name);
@@ -187,9 +189,9 @@ static void php_dom_iterator_move_forward(zend_object_iterator *iter TSRMLS_DC)
 	dom_object *intern;
 	dom_object *nnmap;
 	dom_nnodemap_object *objmap;
-	int ret, previndex=1;
+	int ret, previndex=0;
 	HashTable *nodeht;
-	pval **entry;
+	zval **entry;
 
 	php_dom_iterator *iterator = (php_dom_iterator *)iter;
 
@@ -200,7 +202,8 @@ static void php_dom_iterator_move_forward(zend_object_iterator *iter TSRMLS_DC)
 	curobj = iterator->curobj;
 	intern = (dom_object *)zend_object_store_get_object(curobj TSRMLS_CC);
 	if (intern != NULL && intern->ptr != NULL) {
-		if (objmap->ht == NULL) {
+		if (objmap->nodetype != XML_ENTITY_NODE && 
+			objmap->nodetype != XML_NOTATION_NODE) {
 			if (objmap->nodetype == DOM_NODESET) {
 				nodeht = HASH_OF(objmap->baseobjptr);
 				zend_hash_move_forward(nodeht);
@@ -210,15 +213,19 @@ static void php_dom_iterator_move_forward(zend_object_iterator *iter TSRMLS_DC)
 				}
 			} else {
 				curnode = (xmlNodePtr)((php_libxml_node_ptr *)intern->ptr)->node;
-				if (objmap->nodetype == XML_ATTRIBUTE_NODE || objmap->nodetype == XML_ELEMENT_NODE) {
+				if (objmap->nodetype == XML_ATTRIBUTE_NODE || 
+					objmap->nodetype == XML_ELEMENT_NODE) {
 					curnode = curnode->next;
 				} else {
 					/* Nav the tree evey time as this is LIVE */
 					basenode = dom_object_get_node(objmap->baseobj);
-					if (basenode && (basenode->type == XML_DOCUMENT_NODE || basenode->type == XML_HTML_DOCUMENT_NODE)) {
+					if (basenode && (basenode->type == XML_DOCUMENT_NODE || 
+						basenode->type == XML_HTML_DOCUMENT_NODE)) {
 						basenode = xmlDocGetRootElement((xmlDoc *) basenode);
-					} else {
+					} else if (basenode) {
 						basenode = basenode->children;
+					} else {
+						goto err;
 					}
 					curnode = dom_get_elements_by_tag_name_ns_raw(basenode, objmap->ns, objmap->local, &previndex, iter->index);
 				}
@@ -231,7 +238,7 @@ static void php_dom_iterator_move_forward(zend_object_iterator *iter TSRMLS_DC)
 			}
 		}
 	}
-
+err:
 	zval_ptr_dtor((zval**)&curobj);
 	if (curnode) {
 		MAKE_STD_ZVAL(curattr);
@@ -250,7 +257,7 @@ zend_object_iterator_funcs php_dom_iterator_funcs = {
 	NULL
 };
 
-zend_object_iterator *php_dom_get_iterator(zend_class_entry *ce, zval *object TSRMLS_DC)
+zend_object_iterator *php_dom_get_iterator(zend_class_entry *ce, zval *object, int by_ref TSRMLS_DC)
 {
 	dom_object *intern;
 	dom_nnodemap_object *objmap;
@@ -258,9 +265,13 @@ zend_object_iterator *php_dom_get_iterator(zend_class_entry *ce, zval *object TS
 	zval *curattr = NULL;
 	int ret, curindex = 0;
 	HashTable *nodeht;
-	pval **entry;
+	zval **entry;
+	php_dom_iterator *iterator;
 
-	php_dom_iterator *iterator = emalloc(sizeof(php_dom_iterator));
+	if (by_ref) {
+		zend_error(E_ERROR, "An iterator cannot be used with foreach by reference");
+	}
+	iterator = emalloc(sizeof(php_dom_iterator));
 
 	object->refcount++;
 	iterator->intern.data = (void*)object;
@@ -269,7 +280,8 @@ zend_object_iterator *php_dom_get_iterator(zend_class_entry *ce, zval *object TS
 	intern = (dom_object *)zend_object_store_get_object(object TSRMLS_CC);
 	objmap = (dom_nnodemap_object *)intern->ptr;
 	if (objmap != NULL) {
-		if (objmap->ht == NULL) {
+		if (objmap->nodetype != XML_ENTITY_NODE && 
+			objmap->nodetype != XML_NOTATION_NODE) {
 			if (objmap->nodetype == DOM_NODESET) {
 				nodeht = HASH_OF(objmap->baseobjptr);
 				zend_hash_internal_pointer_reset(nodeht);
@@ -279,6 +291,9 @@ zend_object_iterator *php_dom_get_iterator(zend_class_entry *ce, zval *object TS
 				}
 			} else {
 				nodep = (xmlNode *)dom_object_get_node(objmap->baseobj);
+				if (!nodep) {
+					goto err;
+				}
 				if (objmap->nodetype == XML_ATTRIBUTE_NODE || objmap->nodetype == XML_ELEMENT_NODE) {
 					if (objmap->nodetype == XML_ATTRIBUTE_NODE) {
 						curnode = (xmlNodePtr) nodep->properties;
@@ -295,10 +310,14 @@ zend_object_iterator *php_dom_get_iterator(zend_class_entry *ce, zval *object TS
 				}
 			}
 		} else {
-			curnode = php_dom_libxml_hash_iter(objmap->ht, 0);
+			if (objmap->nodetype == XML_ENTITY_NODE) {
+				curnode = php_dom_libxml_hash_iter(objmap->ht, 0);
+			} else {
+				curnode = php_dom_libxml_notation_iter(objmap->ht, 0);
+			}
 		}
 	}
-
+err:
 	if (curnode) {
 		MAKE_STD_ZVAL(curattr);
 		curattr = php_dom_create_object(curnode, &ret, NULL, curattr, objmap->baseobj TSRMLS_CC);
