@@ -17,7 +17,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: logical_filters.c,v 1.1.2.24 2008/03/18 23:32:42 iliaa Exp $ */
+/* $Id: logical_filters.c,v 1.1.2.25 2008/10/20 23:23:45 iliaa Exp $ */
 
 #include "php_filter.h"
 #include "filter_private.h"
@@ -529,6 +529,7 @@ static int _php_filter_validate_ipv6(char *str, int str_len TSRMLS_DC) /* {{{ */
 	char *ipv4;
 	char *end;
 	int ip4elm[4];
+	char *s = str;
 
 	if (!memchr(str, ':', str_len)) {
 		return 0;
@@ -568,6 +569,8 @@ static int _php_filter_validate_ipv6(char *str, int str_len TSRMLS_DC) /* {{{ */
 					return 1;
 				}
 				compressed = 1;
+			} else if ((str - 1) == s) {
+				return 0;
 			}				
 		}
 		n = 0;
