@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: ibase_query.c,v 1.23.2.1.2.11 2007/12/31 07:20:07 sebastian Exp $ */
+/* $Id: ibase_query.c,v 1.23.2.1.2.12 2008/10/06 14:37:13 felipe Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -1819,16 +1819,17 @@ PHP_FUNCTION(ibase_execute)
 		if (bind_n != expected_n) {
 			php_error_docref(NULL TSRMLS_CC, (bind_n < expected_n) ? E_WARNING : E_NOTICE,
 				"Statement expects %d arguments, %d given", expected_n, bind_n);
+
 			if (bind_n < expected_n) {
 				break;
 			}
-
-		} else if (bind_n > 0) { /* have variables to bind */
-			args = (zval ***) do_alloca(ZEND_NUM_ARGS() * sizeof(zval **));
+		}
+		
+		/* have variables to bind */
+		args = (zval ***) do_alloca((expected_n + 1) * sizeof(zval **));
 	
-			if (FAILURE == zend_get_parameters_array_ex(ZEND_NUM_ARGS(), args)) {
-				break;
-			}
+		if (FAILURE == zend_get_parameters_array_ex((expected_n + 1), args)) {
+			break;
 		}
 
 		/* Have we used this cursor before and it's still open (exec proc has no cursor) ? */
