@@ -331,12 +331,30 @@ mbfl_buffer_converter_feed_result(mbfl_buffer_converter *convd, mbfl_string *str
 	return mbfl_memory_device_result(&convd->device, result);
 }
 
+int mbfl_buffer_illegalchars(mbfl_buffer_converter *convd)
+{
+	int num_illegalchars = 0;
+
+	if (convd == NULL) {
+		return 0;
+	}
+
+	if (convd->filter1 != NULL) {
+		num_illegalchars += convd->filter1->num_illegalchar;
+	}
+
+	if (convd->filter2 != NULL) {
+		num_illegalchars += convd->filter2->num_illegalchar;
+	}
+
+	return (num_illegalchars);
+}
 
 /*
  * encoding detector
  */
 mbfl_encoding_detector *
-mbfl_encoding_detector_new(enum mbfl_no_encoding *elist, int elistsz)
+mbfl_encoding_detector_new(enum mbfl_no_encoding *elist, int elistsz, int strict)
 {
 	mbfl_encoding_detector *identd;
 
@@ -371,6 +389,9 @@ mbfl_encoding_detector_new(enum mbfl_no_encoding *elist, int elistsz)
 	}
 	identd->filter_list_size = num;
 
+	/* set strict flag */
+	identd->strict = strict;
+
 	return identd;
 }
 
@@ -441,10 +462,24 @@ enum mbfl_no_encoding mbfl_encoding_detector_judge(mbfl_encoding_detector *ident
 		while (n >= 0) {
 			filter = identd->filter_list[n];
 			if (!filter->flag) {
+				if (identd->strict && filter->status) {
+					continue;
+				}
 				encoding = filter->encoding->no_encoding;
 			}
 			n--;
 		}
+
+		if (encoding ==	mbfl_no_encoding_invalid) {
+			n = identd->filter_list_size - 1;
+			while (n >= 0) {
+				filter = identd->filter_list[n];
+				if (!filter->flag) {
+					encoding = filter->encoding->no_encoding;
+				}
+				n--;
+			}
+		}
 	}
 
 	return encoding;
@@ -576,11 +611,25 @@ mbfl_identify_encoding(mbfl_string *string, enum mbfl_no_encoding *elist, int el
 	for (i = 0; i < num; i++) {
 		filter = &flist[i];
 		if (!filter->flag) {
+			if (strict && filter->status) {
+				continue;
+			}
 			encoding = filter->encoding;
 			break;
 		}
 	}
 
+	/* fall-back judge */
+	if (!encoding) {
+		for (i = 0; i < num; i++) {
+			filter = &flist[i];
+			if (!filter->flag) {
+				encoding = filter->encoding;
+				break;
+			}
+		}
+	}
+
 	/* cleanup */
 	/* dtors should be called in reverse order */
 	i = num; while (--i >= 0) {
@@ -608,11 +657,11 @@ mbfl_identify_encoding_name(mbfl_string *string, enum mbfl_no_encoding *elist, i
 }
 
 enum mbfl_no_encoding
-mbfl_identify_encoding_no(mbfl_string *string, enum mbfl_no_encoding *elist, int elistsz)
+mbfl_identify_encoding_no(mbfl_string *string, enum mbfl_no_encoding *elist, int elistsz, int strict)
 {
 	const mbfl_encoding *encoding;
 
-	encoding = mbfl_identify_encoding(string, elist, elistsz, 0);
+	encoding = mbfl_identify_encoding(string, elist, elistsz, strict);
 	if (encoding != NULL &&
 	    encoding->no_encoding > mbfl_no_encoding_charset_min &&
 	    encoding->no_encoding < mbfl_no_encoding_charset_max) {
@@ -809,7 +858,7 @@ mbfl_strpos(
     int offset,
     int reverse)
 {
-	int n, result;
+	int n, result, negative_offset = 0;
 	unsigned char *p;
 	mbfl_convert_filter *filter;
 	struct collector_strpos_data pc;
@@ -855,6 +904,12 @@ mbfl_strpos(
 		mbfl_wchar_device_clear(&pc.needle);
 		return -4;
 	}
+
+	if (offset < 0) {
+		negative_offset = -offset-1;
+		offset = 0;
+	}
+
 	pc.start = offset;
 	pc.output = 0;
 	pc.needle_pos = 0;
@@ -863,7 +918,7 @@ mbfl_strpos(
 
 	/* feed data */
 	p = haystack->val;
-	n = haystack->len;
+	n = haystack->len - negative_offset;
 	if (p != NULL) {
 		while (n > 0) {
 			if ((*filter->filter_function)(*p++, filter) < 0) {
@@ -958,6 +1013,7 @@ mbfl_substr_count(
 			if (pc.matched_pos >= 0) {
 				++result;
 				pc.matched_pos = -1;
+				pc.needle_pos = 0;
 			}
 			n--;
 		}
