@@ -1,5 +1,5 @@
 /**********************************************************************
-  iso8859_15.c -  Oniguruma (regular expression library)
+  iso8859_7.c -  Oniguruma (regular expression library)
 **********************************************************************/
 /*-
  * Copyright (c) 2002-2005  K.Kosako  <sndgk393 AT ybb DOT ne DOT jp>
@@ -29,11 +29,11 @@
 
 #include "regenc.h"
 
-#define ENC_ISO_8859_15_TO_LOWER_CASE(c) EncISO_8859_15_ToLowerCaseTable[c]
-#define ENC_IS_ISO_8859_15_CTYPE(code,ctype) \
-  ((EncISO_8859_15_CtypeTable[code] & ctype) != 0)
+#define ENC_ISO_8859_7_TO_LOWER_CASE(c) EncISO_8859_7_ToLowerCaseTable[c]
+#define ENC_IS_ISO_8859_7_CTYPE(code,ctype) \
+  ((EncISO_8859_7_CtypeTable[code] & ctype) != 0)
 
-static const UChar EncISO_8859_15_ToLowerCaseTable[256] = {
+static const UChar EncISO_8859_7_ToLowerCaseTable[256] = {
   '\000', '\001', '\002', '\003', '\004', '\005', '\006', '\007',
   '\010', '\011', '\012', '\013', '\014', '\015', '\016', '\017',
   '\020', '\021', '\022', '\023', '\024', '\025', '\026', '\027',
@@ -54,21 +54,21 @@ static const UChar EncISO_8859_15_ToLowerCaseTable[256] = {
   '\210', '\211', '\212', '\213', '\214', '\215', '\216', '\217',
   '\220', '\221', '\222', '\223', '\224', '\225', '\226', '\227',
   '\230', '\231', '\232', '\233', '\234', '\235', '\236', '\237',
-  '\240', '\241', '\242', '\243', '\244', '\245', '\250', '\247',
+  '\240', '\241', '\242', '\243', '\244', '\245', '\246', '\247',
   '\250', '\251', '\252', '\253', '\254', '\255', '\256', '\257',
-  '\260', '\261', '\262', '\263', '\270', '\265', '\266', '\267',
-  '\270', '\271', '\272', '\273', '\275', '\275', '\377', '\277',
-  '\340', '\341', '\342', '\343', '\344', '\345', '\346', '\347',
+  '\260', '\261', '\262', '\263', '\264', '\265', '\334', '\267',
+  '\335', '\336', '\337', '\273', '\374', '\275', '\375', '\376',
+  '\300', '\341', '\342', '\343', '\344', '\345', '\346', '\347',
   '\350', '\351', '\352', '\353', '\354', '\355', '\356', '\357',
-  '\360', '\361', '\362', '\363', '\364', '\365', '\366', '\327',
-  '\370', '\371', '\372', '\373', '\374', '\375', '\376', '\337',
+  '\360', '\361', '\322', '\363', '\364', '\365', '\366', '\367',
+  '\370', '\371', '\372', '\373', '\334', '\335', '\336', '\337',
   '\340', '\341', '\342', '\343', '\344', '\345', '\346', '\347',
   '\350', '\351', '\352', '\353', '\354', '\355', '\356', '\357',
   '\360', '\361', '\362', '\363', '\364', '\365', '\366', '\367',
   '\370', '\371', '\372', '\373', '\374', '\375', '\376', '\377'
 };
 
-static const unsigned short EncISO_8859_15_CtypeTable[256] = {
+static const unsigned short EncISO_8859_7_CtypeTable[256] = {
   0x2008, 0x2008, 0x2008, 0x2008, 0x2008, 0x2008, 0x2008, 0x2008,
   0x2008, 0x220c, 0x2209, 0x2208, 0x2208, 0x2208, 0x2008, 0x2008,
   0x2008, 0x2008, 0x2008, 0x2008, 0x2008, 0x2008, 0x2008, 0x2008,
@@ -89,41 +89,31 @@ static const unsigned short EncISO_8859_15_CtypeTable[256] = {
   0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008,
   0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008,
   0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008,
-  0x0284, 0x01a0, 0x00a0, 0x00a0, 0x00a0, 0x00a0, 0x14a2, 0x00a0,
-  0x10e2, 0x00a0, 0x10e2, 0x01a0, 0x00a0, 0x01a0, 0x00a0, 0x00a0,
-  0x00a0, 0x00a0, 0x10a0, 0x10a0, 0x14a2, 0x10e2, 0x00a0, 0x01a0,
-  0x10e2, 0x10a0, 0x10e2, 0x01a0, 0x14a2, 0x10e2, 0x14a2, 0x01a0,
+  0x0284, 0x01a0, 0x01a0, 0x00a0, 0x0000, 0x0000, 0x00a0, 0x00a0,
+  0x00a0, 0x00a0, 0x0000, 0x01a0, 0x00a0, 0x01a0, 0x0000, 0x01a0,
+  0x00a0, 0x00a0, 0x10a0, 0x10a0, 0x00a0, 0x00a0, 0x14a2, 0x01a0,
+  0x14a2, 0x14a2, 0x14a2, 0x01a0, 0x14a2, 0x10a0, 0x14a2, 0x14a2,
+  0x10e2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2,
   0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2,
-  0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2,
-  0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x00a0,
-  0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x10e2,
+  0x14a2, 0x14a2, 0x0000, 0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x14a2,
+  0x14a2, 0x14a2, 0x14a2, 0x14a2, 0x10e2, 0x10e2, 0x10e2, 0x10e2,
   0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2,
   0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2,
-  0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x00a0,
-  0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2
+  0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2,
+  0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x10e2, 0x0000
 };
 
 static int
-iso_8859_15_mbc_to_normalize(OnigAmbigType flag,
-			     const UChar** pp, const UChar* end, UChar* lower)
+iso_8859_7_mbc_to_normalize(OnigAmbigType flag,
+                            const UChar** pp, const UChar* end, UChar* lower)
 {
   const UChar* p = *pp;
 
-  if (end > p + 1 && (flag & ONIGENC_AMBIGUOUS_MATCH_COMPOUND) != 0) {
-    if ((*p == 's' && *(p+1) == 's') ||
-	((flag & ONIGENC_AMBIGUOUS_MATCH_ASCII_CASE) != 0 &&
-	 (*p == 'S' && *(p+1) == 'S'))) {
-      *lower = 0xdf;
-      (*pp) += 2;
-      return 1;
-    }
-  }
-
   if (((flag & ONIGENC_AMBIGUOUS_MATCH_ASCII_CASE) != 0 &&
        ONIGENC_IS_MBC_ASCII(p)) ||
       ((flag & ONIGENC_AMBIGUOUS_MATCH_NONASCII_CASE) != 0 &&
        !ONIGENC_IS_MBC_ASCII(p))) {
-    *lower = ENC_ISO_8859_15_TO_LOWER_CASE(*p);
+    *lower = ENC_ISO_8859_7_TO_LOWER_CASE(*p);
   }
   else {
     *lower = *p;
@@ -133,38 +123,21 @@ iso_8859_15_mbc_to_normalize(OnigAmbigType flag,
 }
 
 static int
-iso_8859_15_is_mbc_ambiguous(OnigAmbigType flag,
-			     const UChar** pp, const UChar* end)
+iso_8859_7_is_mbc_ambiguous(OnigAmbigType flag,
+			    const UChar** pp, const UChar* end)
 {
   const UChar* p = *pp;
 
-  if ((flag & ONIGENC_AMBIGUOUS_MATCH_COMPOUND) != 0) {
-    if (end > p + 1) {
-      if ((*p == 's' && *(p+1) == 's') ||
-	  ((flag & ONIGENC_AMBIGUOUS_MATCH_ASCII_CASE) != 0 &&
-	   (*p == 'S' && *(p+1) == 'S'))) {
-	(*pp) += 2;
-	return TRUE;
-      }
-    }
-
-    if (*p == 0xdf) {
-      (*pp)++;
-      return TRUE;
-    }
-  }
-
   (*pp)++;
   if (((flag & ONIGENC_AMBIGUOUS_MATCH_ASCII_CASE) != 0 &&
        ONIGENC_IS_MBC_ASCII(p)) ||
       ((flag & ONIGENC_AMBIGUOUS_MATCH_NONASCII_CASE) != 0 &&
        !ONIGENC_IS_MBC_ASCII(p))) {
-    int v = (EncISO_8859_15_CtypeTable[*p] &
+    int v = (EncISO_8859_7_CtypeTable[*p] &
              (ONIGENC_CTYPE_UPPER | ONIGENC_CTYPE_LOWER));
 
     if ((v | ONIGENC_CTYPE_LOWER) != 0) {
-      /* 0xdf etc.. are lower case letter, but can't convert. */
-      if (*p == 0xdf || *p == 0xaa || *p == 0xb5 || *p == 0xba)
+      if (*p == 0xc0 || *p == 0xe0)
         return FALSE;
       else
         return TRUE;
@@ -176,29 +149,27 @@ iso_8859_15_is_mbc_ambiguous(OnigAmbigType flag,
 }
 
 static int
-iso_8859_15_is_code_ctype(OnigCodePoint code, unsigned int ctype)
+iso_8859_7_is_code_ctype(OnigCodePoint code, unsigned int ctype)
 {
   if (code < 256)
-    return ENC_IS_ISO_8859_15_CTYPE(code, ctype);
+    return ENC_IS_ISO_8859_7_CTYPE(code, ctype);
   else
     return FALSE;
 }
 
 static int
-iso_8859_15_get_all_pair_ambig_codes(OnigAmbigType flag,
-                                     const OnigPairAmbigCodes** ccs)
+iso_8859_7_get_all_pair_ambig_codes(OnigAmbigType flag,
+                                    const OnigPairAmbigCodes** ccs)
 {
   static const OnigPairAmbigCodes cc[] = {
-    { 0xa6, 0xa8 },
-    { 0xa8, 0xa6 },
+    { 0xb6, 0xdc },
+    { 0xb8, 0xdd },
+    { 0xb9, 0xde },
+    { 0xba, 0xdf },
+    { 0xbc, 0xfc },
+    { 0xbe, 0xfd },
+    { 0xbf, 0xfe },
 
-    { 0xb4, 0xb8 },
-    { 0xb8, 0xb4 },
-    { 0xbc, 0xbd },
-    { 0xbd, 0xbc },
-    { 0xbe, 0xff },
-
-    { 0xc0, 0xe0 },
     { 0xc1, 0xe1 },
     { 0xc2, 0xe2 },
     { 0xc3, 0xe3 },
@@ -222,15 +193,16 @@ iso_8859_15_get_all_pair_ambig_codes(OnigAmbigType flag,
     { 0xd4, 0xf4 },
     { 0xd5, 0xf5 },
     { 0xd6, 0xf6 },
+    { 0xd7, 0xf7 },
     { 0xd8, 0xf8 },
     { 0xd9, 0xf9 },
     { 0xda, 0xfa },
     { 0xdb, 0xfb },
-    { 0xdc, 0xfc },
-    { 0xdd, 0xfd },
-    { 0xde, 0xfe },
+    { 0xdc, 0xb6 },
+    { 0xdd, 0xb8 },
+    { 0xde, 0xb9 },
+    { 0xdf, 0xba },
 
-    { 0xe0, 0xc0 },
     { 0xe1, 0xc1 },
     { 0xe2, 0xc2 },
     { 0xe3, 0xc3 },
@@ -254,14 +226,14 @@ iso_8859_15_get_all_pair_ambig_codes(OnigAmbigType flag,
     { 0xf4, 0xd4 },
     { 0xf5, 0xd5 },
     { 0xf6, 0xd6 },
+    { 0xf7, 0xd7 },
     { 0xf8, 0xd8 },
     { 0xf9, 0xd9 },
     { 0xfa, 0xda },
     { 0xfb, 0xdb },
-    { 0xfc, 0xdc },
-    { 0xfd, 0xdd },
-    { 0xfe, 0xde },
-    { 0xff, 0xbe }
+    { 0xfc, 0xbc },
+    { 0xfd, 0xbe },
+    { 0xfe, 0xbf }
   };
 
   if (flag == ONIGENC_AMBIGUOUS_MATCH_ASCII_CASE) {
@@ -276,14 +248,13 @@ iso_8859_15_get_all_pair_ambig_codes(OnigAmbigType flag,
     return 0;
 }
 
-OnigEncodingType OnigEncodingISO_8859_15 = {
+OnigEncodingType OnigEncodingISO_8859_7 = {
   onigenc_single_byte_mbc_enc_len,
-  "ISO-8859-15",  /* name */
+  "ISO-8859-7",  /* name */
   1,             /* max enc length */
   1,             /* min enc length */
   (ONIGENC_AMBIGUOUS_MATCH_ASCII_CASE |
-   ONIGENC_AMBIGUOUS_MATCH_NONASCII_CASE |
-   ONIGENC_AMBIGUOUS_MATCH_COMPOUND),
+   ONIGENC_AMBIGUOUS_MATCH_NONASCII_CASE ),
   {
       (OnigCodePoint )'\\'                       /* esc */
     , (OnigCodePoint )ONIG_INEFFECTIVE_META_CHAR /* anychar '.'  */
@@ -296,11 +267,11 @@ OnigEncodingType OnigEncodingISO_8859_15 = {
   onigenc_single_byte_mbc_to_code,
   onigenc_single_byte_code_to_mbclen,
   onigenc_single_byte_code_to_mbc,
-  iso_8859_15_mbc_to_normalize,
-  iso_8859_15_is_mbc_ambiguous,
-  iso_8859_15_get_all_pair_ambig_codes,
-  onigenc_ess_tsett_get_all_comp_ambig_codes,
-  iso_8859_15_is_code_ctype,
+  iso_8859_7_mbc_to_normalize,
+  iso_8859_7_is_mbc_ambiguous,
+  iso_8859_7_get_all_pair_ambig_codes,
+  onigenc_nothing_get_all_comp_ambig_codes,
+  iso_8859_7_is_code_ctype,
   onigenc_not_support_get_ctype_code_range,
   onigenc_single_byte_left_adjust_char_head,
   onigenc_always_true_is_allowed_reverse_match
