@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: sqlite_driver.c,v 1.20.2.5.2.3 2007/12/31 07:20:10 sebastian Exp $ */
+/* $Id: sqlite_driver.c,v 1.20.2.5.2.4 2008/12/02 21:02:44 felipe Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -368,8 +368,8 @@ static int do_callback(struct pdo_sqlite_fci *fc, zval *cb,
 	}
 
 	/* clean up the params */
-	if (argc) {
-		for (i = is_agg; i < argc; i++) {
+	if (zargs) {
+		for (i = is_agg; i < fake_argc; i++) {
 			zval_ptr_dtor(zargs[i]);
 			efree(zargs[i]);
 		}
@@ -498,6 +498,7 @@ static PHP_METHOD(SQLite, sqliteCreateFunction)
 		MAKE_STD_ZVAL(func->func);
 		*(func->func) = *callback;
 		zval_copy_ctor(func->func);
+		INIT_PZVAL(func->func);
 		
 		func->argc = argc;
 
@@ -576,10 +577,12 @@ static PHP_METHOD(SQLite, sqliteCreateAggregate)
 		MAKE_STD_ZVAL(func->step);
 		*(func->step) = *step_callback;
 		zval_copy_ctor(func->step);
+		INIT_PZVAL(func->step);
 
 		MAKE_STD_ZVAL(func->fini);
 		*(func->fini) = *fini_callback;
 		zval_copy_ctor(func->fini);
+		INIT_PZVAL(func->fini);
 		
 		func->argc = argc;
 
