@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2008 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: posix.c,v 1.70.2.3.2.16 2007/07/25 09:06:22 bjori Exp $ */
+/* $Id: posix.c,v 1.70.2.3.2.20 2008/09/20 22:12:59 lbarnaud Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -147,7 +147,7 @@ zend_function_entry posix_functions[] = {
 static PHP_MINFO_FUNCTION(posix)
 {
 	php_info_print_table_start();
-	php_info_print_table_row(2, "Revision", "$Revision: 1.70.2.3.2.16 $");
+	php_info_print_table_row(2, "Revision", "$Revision: 1.70.2.3.2.20 $");
 	php_info_print_table_end();
 }
 /* }}} */
@@ -467,7 +467,7 @@ PHP_FUNCTION(posix_uname)
 	add_assoc_string(return_value, "release",  u.release,  1);
 	add_assoc_string(return_value, "version",  u.version,  1);
 	add_assoc_string(return_value, "machine",  u.machine,  1);
-#ifdef _GNU_SOURCE
+#if defined(_GNU_SOURCE) && !defined(DARWIN) && defined(HAVE_UTSNAME_DOMAINNAME)
 	add_assoc_string(return_value, "domainname", u.domainname, 1);
 #endif
 }
@@ -875,7 +875,7 @@ PHP_FUNCTION(posix_getgrgid)
 #if defined(ZTS) && defined(HAVE_GETGRGID_R) && defined(_SC_GETGR_R_SIZE_MAX)
 	int ret;
 	struct group _g;
-	struct group *retgrptr;
+	struct group *retgrptr = NULL;
 	long grbuflen;
 	char *grbuf;
 #endif
@@ -894,7 +894,7 @@ PHP_FUNCTION(posix_getgrgid)
 	grbuf = emalloc(grbuflen);
 
 	ret = getgrgid_r(gid, &_g, grbuf, grbuflen, &retgrptr);
-	if (ret) {
+	if (ret || retgrptr == NULL) {
 		POSIX_G(last_error) = ret;
 		efree(grbuf);
 		RETURN_FALSE;
@@ -1009,7 +1009,7 @@ PHP_FUNCTION(posix_getpwuid)
 	pwbuf = emalloc(pwbuflen);
 
 	ret = getpwuid_r(uid, &_pw, pwbuf, pwbuflen, &retpwptr);
-	if (ret) {
+	if (ret || retpwptr == NULL) {
 		POSIX_G(last_error) = ret;
 		efree(pwbuf);
 		RETURN_FALSE;
