@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2008 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: pspell.c,v 1.45.2.4.2.7 2007/02/24 02:17:25 helly Exp $ */
+/* $Id: pspell.c,v 1.45.2.4.2.9 2008/08/13 08:09:55 tony2001 Exp $ */
 
 #define IS_EXT_MODULE
 
@@ -251,7 +251,7 @@ static PHP_FUNCTION(pspell_new)
 
 	if(pspell_error_number(ret) != 0){
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "PSPELL couldn't open the dictionary. reason: %s ", pspell_error_message(ret));
-		delete_pspell_manager(ret);
+		delete_pspell_can_have_error(ret);
 		RETURN_FALSE;
 	}
 	
@@ -374,7 +374,7 @@ static PHP_FUNCTION(pspell_new_personal)
 
 	if(pspell_error_number(ret) != 0){
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "PSPELL couldn't open the dictionary. reason: %s ", pspell_error_message(ret));
-		delete_pspell_manager(ret);
+		delete_pspell_can_have_error(ret);
 		RETURN_FALSE;
 	}
 	
@@ -408,7 +408,7 @@ static PHP_FUNCTION(pspell_new_config)
 
 	if(pspell_error_number(ret) != 0){
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "PSPELL couldn't open the dictionary. reason: %s ", pspell_error_message(ret));
-		delete_pspell_manager(ret);
+		delete_pspell_can_have_error(ret);
 		RETURN_FALSE;
 	}
 	
