@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: session.c,v 1.417.2.8.2.42 2008/04/29 14:42:38 scottmac Exp $ */
+/* $Id: session.c,v 1.417.2.8.2.44 2008/08/06 21:28:38 kalle Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -1509,6 +1509,15 @@ PHP_FUNCTION(session_set_save_handler)
 
 	zend_alter_ini_entry("session.save_handler", sizeof("session.save_handler"), "user", sizeof("user")-1, PHP_INI_USER, PHP_INI_STAGE_RUNTIME);
 
+	mdata = PS(mod_data);
+
+	if (mdata) {
+		for (i = 0; i < 6; i++) {
+			zval_ptr_dtor(&mdata->names[i]);
+		}
+		efree(mdata);
+	}
+
 	mdata = emalloc(sizeof(*mdata));
 
 	for (i = 0; i < 6; i++) {
@@ -1916,6 +1925,21 @@ PHP_RINIT_FUNCTION(session)
 		}
 	}
 
+	if (PS(serializer) == NULL) {
+		char *value;
+
+		value = zend_ini_string("session.serialize_handler", sizeof("session.serialize_handler"), 0);
+		if (value) {
+			PS(serializer) = _php_find_ps_serializer(value TSRMLS_CC);
+		}
+	}
+
+	if (PS(mod) == NULL || PS(serializer) == NULL) {
+		/* current status is unusable */
+		PS(session_status) = php_session_disabled;
+		return SUCCESS;
+	}
+
 	if (PS(auto_start)) {
 		php_session_start(TSRMLS_C);
 	}
