@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2008 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,13 +16,16 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: spl_iterators.h,v 1.18.2.7 2006/03/05 17:39:49 helly Exp $ */
+/* $Id: spl_iterators.h,v 1.18.2.7.2.14 2007/12/31 07:20:11 sebastian Exp $ */
 
 #ifndef SPL_ITERATORS_H
 #define SPL_ITERATORS_H
 
 #include "php.h"
 #include "php_spl.h"
+#if HAVE_PCRE || HAVE_BUNDLED_PCRE
+#include "ext/pcre/php_pcre.h"
+#endif
 
 #define spl_ce_Traversable   zend_ce_traversable
 #define spl_ce_Iterator      zend_ce_iterator
@@ -45,11 +48,15 @@ extern PHPAPI zend_class_entry *spl_ce_NoRewindIterator;
 extern PHPAPI zend_class_entry *spl_ce_InfiniteIterator;
 extern PHPAPI zend_class_entry *spl_ce_EmptyIterator;
 extern PHPAPI zend_class_entry *spl_ce_AppendIterator;
+extern PHPAPI zend_class_entry *spl_ce_RegexIterator;
+extern PHPAPI zend_class_entry *spl_ce_RecursiveRegexIterator;
+extern PHPAPI zend_class_entry *spl_ce_Countable;
 
 PHP_MINIT_FUNCTION(spl_iterators);
 
 PHP_FUNCTION(iterator_to_array);
 PHP_FUNCTION(iterator_count);
+PHP_FUNCTION(iterator_apply);
 
 typedef enum {
 	DIT_Default = 0,
@@ -63,21 +70,42 @@ typedef enum {
 	DIT_NoRewindIterator,
 	DIT_InfiniteIterator,
 	DIT_AppendIterator,
+#if HAVE_PCRE || HAVE_BUNDLED_PCRE
+	DIT_RegexIterator,
+	DIT_RecursiveRegexIterator,
+#endif
 	DIT_Unknown = ~0
 } dual_it_type;
 
 enum {
 	/* public */
 	CIT_CALL_TOSTRING        = 0x00000001,
-	CIT_CATCH_GET_CHILD      = 0x00000002,
-	CIT_TOSTRING_USE_KEY     = 0x00000010,
-	CIT_TOSTRING_USE_CURRENT = 0x00000020,
+	CIT_TOSTRING_USE_KEY     = 0x00000002,
+	CIT_TOSTRING_USE_CURRENT = 0x00000004,
+	CIT_TOSTRING_USE_INNER   = 0x00000008,
+	CIT_CATCH_GET_CHILD      = 0x00000010,
+	CIT_FULL_CACHE           = 0x00000100,
 	CIT_PUBLIC               = 0x0000FFFF,
 	/* private */
 	CIT_VALID                = 0x00010000,
 	CIT_HAS_CHILDREN         = 0x00020000
 };
 
+enum {
+	/* public */
+	REGIT_USE_KEY            = 0x00000001,
+	REGIT_INVERTED           = 0x00000002
+};
+
+typedef enum {
+	REGIT_MODE_MATCH,
+	REGIT_MODE_GET_MATCH,
+	REGIT_MODE_ALL_MATCHES,
+	REGIT_MODE_SPLIT,
+	REGIT_MODE_REPLACE,
+	REGIT_MODE_MAX
+} regex_mode;
+
 typedef struct _spl_dual_it_object {
 	zend_object              std;
 	struct {
@@ -101,17 +129,32 @@ typedef struct _spl_dual_it_object {
 			long             count;
 		} limit;
 		struct {
-			int              flags; /* CIT_* */
+			long             flags; /* CIT_* */
 			zval             *zstr;
 			zval             *zchildren;
+			zval             *zcache;
 		} caching;
 		struct {
 			zval                 *zarrayit;
 			zend_object_iterator *iterator;
 		} append;
+#if HAVE_PCRE || HAVE_BUNDLED_PCRE
+		struct {
+			int              use_flags;
+			long             flags;
+			regex_mode       mode;
+			long             preg_flags;
+			pcre_cache_entry *pce;
+			char             *regex;
+		} regex;
+#endif
 	} u;
 } spl_dual_it_object;
 
+typedef int (*spl_iterator_apply_func_t)(zend_object_iterator *iter, void *puser TSRMLS_DC);
+
+PHPAPI int spl_iterator_apply(zval *obj, spl_iterator_apply_func_t apply_func, void *puser TSRMLS_DC);
+
 #endif /* SPL_ITERATORS_H */
 
 /*
