@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2008 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: info.c,v 1.249.2.10.2.11 2007/04/02 12:41:07 sniper Exp $ */
+/* $Id: info.c,v 1.249.2.10.2.18 2008/08/13 01:01:28 jani Exp $ */
 
 #include "php.h"
 #include "php_ini.h"
@@ -57,6 +57,7 @@ ZEND_EXTERN_MODULE_GLOBALS(iconv)
 						} \
 
 PHPAPI extern char *php_ini_opened_path;
+PHPAPI extern char *php_ini_scanned_path;
 PHPAPI extern char *php_ini_scanned_files;
 	
 static int php_info_write_wrapper(const char *str, uint str_length)
@@ -277,7 +278,7 @@ PHPAPI char *php_get_uname(char mode)
 				php_uname = tmp_uname;
 				break;
 			case PROCESSOR_ARCHITECTURE_MIPS :
-				php_uname = "MIPS R4000";
+				snprintf(tmp_uname, sizeof(tmp_uname), "MIPS R%d000", SysInfo.wProcessorLevel);
 				php_uname = tmp_uname;
 				break;
 			case PROCESSOR_ARCHITECTURE_ALPHA :
@@ -326,6 +327,30 @@ PHPAPI char *php_get_uname(char mode)
 	if (uname((struct utsname *)&buf) == -1) {
 		php_uname = PHP_UNAME;
 	} else {
+#ifdef NETWARE
+		if (mode == 's') {
+			php_uname = buf.sysname;
+		} else if (mode == 'r') {
+			snprintf(tmp_uname, sizeof(tmp_uname), "%d.%d.%d", 
+					 buf.netware_major, buf.netware_minor, buf.netware_revision);
+			php_uname = tmp_uname;
+		} else if (mode == 'n') {
+			php_uname = buf.servername;
+		} else if (mode == 'v') {
+			snprintf(tmp_uname, sizeof(tmp_uname), "libc-%d.%d.%d #%d",
+					 buf.libmajor, buf.libminor, buf.librevision, buf.libthreshold);
+			php_uname = tmp_uname;
+		} else if (mode == 'm') {
+			php_uname = buf.machine;
+		} else { /* assume mode == 'a' */
+			snprintf(tmp_uname, sizeof(tmp_uname), "%s %s %d.%d.%d libc-%d.%d.%d #%d %s",
+					 buf.sysname, buf.servername,
+					 buf.netware_major, buf.netware_minor, buf.netware_revision,
+					 buf.libmajor, buf.libminor, buf.librevision, buf.libthreshold,
+					 buf.machine);
+			php_uname = tmp_uname;
+		}
+#else
 		if (mode == 's') {
 			php_uname = buf.sysname;
 		} else if (mode == 'r') {
@@ -342,6 +367,7 @@ PHPAPI char *php_get_uname(char mode)
 					 buf.machine);
 			php_uname = tmp_uname;
 		}
+#endif /* NETWARE */
 	}
 #else
 	php_uname = PHP_UNAME;
@@ -476,14 +502,9 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 
 		php_info_print_table_row(2, "Configuration File (php.ini) Path", PHP_CONFIG_FILE_PATH);
 		php_info_print_table_row(2, "Loaded Configuration File", php_ini_opened_path ? php_ini_opened_path : "(none)");
+		php_info_print_table_row(2, "Scan this dir for additional .ini files", php_ini_scanned_path ? php_ini_scanned_path : "(none)");
+		php_info_print_table_row(2, "additional .ini files parsed", php_ini_scanned_files ? php_ini_scanned_files : "(none)");
 
-		if (strlen(PHP_CONFIG_FILE_SCAN_DIR)) {
-			php_info_print_table_row(2, "Scan this dir for additional .ini files", PHP_CONFIG_FILE_SCAN_DIR);
-			if (php_ini_scanned_files) {
-				php_info_print_table_row(2, "additional .ini files parsed", php_ini_scanned_files);
-			}
-		}
-		
 		snprintf(temp_api, sizeof(temp_api), "%d", PHP_API_VERSION);
 		php_info_print_table_row(2, "PHP API", temp_api);
 
@@ -781,7 +802,7 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 /* }}} */
 
 
-PHPAPI void php_info_print_table_start()
+PHPAPI void php_info_print_table_start(void)
 {
 	if (!sapi_module.phpinfo_as_text) {
 		php_printf("<table border=\"0\" cellpadding=\"3\" width=\"600\">\n");
@@ -790,7 +811,7 @@ PHPAPI void php_info_print_table_start()
 	}	
 }
 
-PHPAPI void php_info_print_table_end()
+PHPAPI void php_info_print_table_end(void)
 {
 	if (!sapi_module.phpinfo_as_text) {
 		php_printf("</table><br />\n");
@@ -814,7 +835,7 @@ PHPAPI void php_info_print_box_start(int flag)
 	}
 }
 
-PHPAPI void php_info_print_box_end()
+PHPAPI void php_info_print_box_end(void)
 {
 	if (!sapi_module.phpinfo_as_text) {
 		php_printf("</td></tr>\n");
@@ -822,7 +843,7 @@ PHPAPI void php_info_print_box_end()
 	php_info_print_table_end();
 }
 
-PHPAPI void php_info_print_hr()
+PHPAPI void php_info_print_hr(void)
 {
 	if (!sapi_module.phpinfo_as_text) {
 		php_printf("<hr />\n");
@@ -1056,7 +1077,7 @@ PHP_FUNCTION(phpcredits)
 
 /* {{{ php_logo_guid
  */
-PHPAPI char *php_logo_guid()
+PHPAPI char *php_logo_guid(void)
 {
 	char *logo_guid;
 
@@ -1066,7 +1087,7 @@ PHPAPI char *php_logo_guid()
 	the_time = time(NULL);
 	ta = php_localtime_r(&the_time, &tmbuf);
 
-	if ((ta->tm_mon==3) && (ta->tm_mday==1)) {
+	if (ta && (ta->tm_mon==3) && (ta->tm_mday==1)) {
 		logo_guid = PHP_EGG_LOGO_GUID;
 	} else {
 		logo_guid = PHP_LOGO_GUID;
@@ -1170,6 +1191,18 @@ PHP_FUNCTION(php_ini_scanned_files)
 }
 /* }}} */
 
+/* {{{ proto string php_ini_loaded_file(void)
+   Return the actual loaded ini filename */
+PHP_FUNCTION(php_ini_loaded_file)
+{
+	if (php_ini_opened_path) {
+		RETURN_STRING(php_ini_opened_path, 1);
+	} else {
+		RETURN_FALSE;
+	}
+}
+/* }}} */
+
 /*
  * Local variables:
  * tab-width: 4
