@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2008 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: mail.c,v 1.87.2.1.2.8 2007/10/04 13:31:11 jani Exp $ */
+/* $Id: mail.c,v 1.87.2.1.2.10 2008/10/03 13:59:33 mkoppanen Exp $ */
 
 #include <stdlib.h>
 #include <ctype.h>
@@ -31,6 +31,12 @@
 #include <sys/sysexits.h>
 #endif
 
+#if PHP_SIGCHILD
+#if HAVE_SIGNAL_H
+#include <signal.h>
+#endif
+#endif
+
 #include "php_mail.h"
 #include "php_ini.h"
 #include "safe_mode.h"
@@ -200,6 +206,9 @@ PHPAPI int php_mail(char *to, char *subject, char *message, char *headers, char
 	int ret;
 	char *sendmail_path = INI_STR("sendmail_path");
 	char *sendmail_cmd = NULL;
+#if PHP_SIGCHILD
+	void (*sig_handler)() = NULL;
+#endif
 
 	if (!sendmail_path) {
 #if (defined PHP_WIN32 || defined NETWARE)
@@ -224,6 +233,16 @@ PHPAPI int php_mail(char *to, char *subject, char *message, char *headers, char
 		sendmail_cmd = sendmail_path;
 	}
 
+#if PHP_SIGCHILD
+	/* Set signal handler of SIGCHLD to default to prevent other signal handlers
+	 * from being called and reaping the return code when our child exits.
+	 * The original handler needs to be restored after pclose() */
+	sig_handler = (void *)signal(SIGCHLD, SIG_DFL);
+	if (sig_handler == SIG_ERR) {
+		sig_handler = NULL;
+	}
+#endif
+
 #ifdef PHP_WIN32
 	sendmail = popen(sendmail_cmd, "wb");
 #else
@@ -241,6 +260,13 @@ PHPAPI int php_mail(char *to, char *subject, char *message, char *headers, char
 		if (EACCES == errno) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Permission denied: unable to execute shell to run mail delivery binary '%s'", sendmail_path);
 			pclose(sendmail);
+#if PHP_SIGCHILD
+			/* Restore handler in case of error on Windows
+			   Not sure if this applicable on Win but just in case. */
+			if (sig_handler) {
+				signal(SIGCHLD, sig_handler);
+			}
+#endif	
 			return 0;
 		}
 #endif
@@ -251,6 +277,12 @@ PHPAPI int php_mail(char *to, char *subject, char *message, char *headers, char
 		}
 		fprintf(sendmail, "\n%s\n", message);
 		ret = pclose(sendmail);
+#if PHP_SIGCHILD
+		if (sig_handler) {
+			signal(SIGCHLD, sig_handler);
+		}
+#endif
+
 #ifdef PHP_WIN32
 		if (ret == -1)
 #else
@@ -269,6 +301,11 @@ PHPAPI int php_mail(char *to, char *subject, char *message, char *headers, char
 		}
 	} else {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not execute mail delivery program '%s'", sendmail_path);
+#if PHP_SIGCHILD
+		if (sig_handler) {
+			signal(SIGCHLD, sig_handler);						
+		}
+#endif
 		return 0;
 	}
 
